<?php

namespace Teknisa\TCO\Services;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\Remote\FileUploader\Server;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;

class Tco01100GeneralLibPropCom extends \Teknisa\GER\Controllers\TeknisaCrud {
    protected $tcoUtil;
    protected $tcoList;
    protected $tcoEmail;
    protected $connection;
    protected $environment;
    protected $crudService;
    protected $entityManager;
    protected $dataSourceName;
    protected $dataSourceManager;

    public function __construct($crudService, ParameterBag $parameterBag, $environment, $entityManager, $dataSourceManager, $newCode, $tcoUtil, $tcoList, $tcoEmail){
        $this->tcoUtil           = $tcoUtil;
        $this->newCode           = $newCode;
        $this->tcoList           = $tcoList;
        $this->tcoEmail          = $tcoEmail;
        $this->connection        = $entityManager->getConnection();
        $this->crudService       = $crudService;
        $this->environment       = $environment;
        $this->entityManager     = $entityManager;
        $this->dataSourceManager = $dataSourceManager;
    }
    
    /**
     * Define a lista no parametro referente usando
     * a classe de lista do TcoUtil.
     * 
     * @param {String|Array} Array com valores de
     *  parametros a serem definidos como lista.
     */
    public function setListOnParam(&$param){
        if($param!='T' && is_array($param)){

            $param = implode('_', $param);
            $param = $this->tcoList->criarListaValores($param);
        } 
    }

    /**
     * Retorna o gerente de negocio de acordo com o código
     * do operador passado por parâmetro.
     * 
     * @param {String} $cdOperador Código do operador.
     * 
     * @return {Array} Gerente de negócio correspondente.
     */
    public function getGerenteNegocio($cdOperador = ''){
        $conditions = array(
            array(
                "columnName" => "CDOPERADOR", 
                "operator"   => "=",
                "value"      => $cdOperador
            )
        );

        $filterCriteria = new FilterCriteria("gerentenegocio", $conditions);
        $dataSet = $this->dataSourceManager->findBy($filterCriteria)->getRows();

        return sizeof($dataSet) ? $dataSet[0] : false;
    }

    /**
     * Retorna o estado de existência e validade
     * do endereço passado por parâmetro.
     * 
     * @param {String} $cdPais      Código do Pais.
     * @param {String} $sgEstado    Código do Estado.
     * @param {String} $cdMunicipio Código do Municipio.
     * 
     * @return {Array} Gerente de negócio correspondente.
     */
    public function isValidEndereco($cdPais, $sgEstado, $cdMunicipio){
        $conditions = array(
            array(
                "columnName" => "CDPAIS", 
                "operator"   => "=",
                "value"      => $cdPais
            )
        );

        $filterCriteria = new FilterCriteria("pais", $conditions);
        $dataSet = $this->dataSourceManager->findBy($filterCriteria)->getRows();
        $validPais = sizeof($dataSet) ? ($dataSet[0]['IDATIVO']=='S' ? true : false) : false;
        
        $conditions[] = array(
            "columnName" => "SGESTADO", 
            "operator"   => "=",
            "value"      => $sgEstado
        );

        $filterCriteria = new FilterCriteria("estado", $conditions);
        $dataSet = $this->dataSourceManager->findBy($filterCriteria)->getRows();
        $validEstado = sizeof($dataSet) ? ($dataSet[0]['IDATIVO']=='S' ? true : false) : false;
        
        $conditions[] = array(
            "columnName" => "CDMUNICIPIO",
            "operator"   => "=",
            "value"      => $cdMunicipio
        );

        $filterCriteria = new FilterCriteria("municipio", $conditions);
        $dataSet = $this->dataSourceManager->findBy($filterCriteria)->getRows();
        $validAddress = sizeof($dataSet) ? ($dataSet[0]['IDATIVO']=='S' ? true : false) : false;

        return $validPais && $validEstado && $validAddress;
    }

    /**
     * Valida se o operador passado por parâmetro é um 
     * responsável válido, ou respectivo lider, pela 
     * proposta comercial.
     * 
     * @param {String} $cdOperador Código do Operador.
     * @param {String} $nrPropostaComercial Número da proposta
     *  comercial.
     */
    public function isValidResponsavelPropCom($cdOperador, $nrPropostaComercial){
        $sql = "SELECT 1
                FROM
                    RESPONSAVELPROPCOM R
                    INNER JOIN TABLE ( LIDERADOGN(:CDOPERADOR) ) L 
                        ON L.NRGERENTENEGOCIO = R.NRGERENTENEGOCIO
                WHERE
                    NRPROPOSTACOMERCIAL = :NRPROPOSTACOMERCIAL";

        $params = array(
            "CDOPERADOR"    => $cdOperador,
            "NRPROPOSTACOMERCIAL" => $nrPropostaComercial
        );

        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        $result = $query->fetchAll();
        
        return (bool)$result;
    }

    /**
     * Retorna os dados do dataSource de acordo com
     * os parametros passados.
     * 
     * @param String $dataSourceName
     * @param Array $params
     */
    public function getData($dataSourceName, $params){
        $conditions = array();

        foreach ($params as $name => $value) {
            $conditions[] = array(
                "columnName" => $name,
                "operator"   => "=",
                "value"      => $value
            );
        }

        $filterCriteria = new FilterCriteria($dataSourceName, $conditions);
        $dataSet = $this->dataSourceManager->findBy($filterCriteria)->getRows();   
        return sizeof($dataSet) ? $dataSet : false;
    }

    /**
     * Retorna todas as informações do detalhe e 
     * do cardapio de trabalho do servico da proposta
     * comercial.
     * 
     * @param int|float $nrServPropCom
     */
    public function getServPropComComplete($nrServPropCom){
        try{
            $sql = "SELECT
                SPC.*,
                SP.NMSERVICOPROPOSTA,
                SP.CDSERVFILI,
                TP.NMTIPOPUBLICO TIPOPUBLICO,
                PQ.NMPADRAOQUALIDADE PADRAOQUALIDADE,
                CT.NRCARDAPIOTRAB,
                CT.CDREGIAO,
                CT.DTCUSTO,
                CT.CDTIPOCUSTO,
                CT.IDORIGEM,
                CT.VRPEACRESCUSTO,
                CUSTO_PERC_CARDTRAB(CT.NRCARDAPIOTRAB, 'ALIMENTAR') CUSTALISERVPROPCOM,
                CUSTO_PERC_CARDTRAB(CT.NRCARDAPIOTRAB, 'DESCARTAVEL') CUSTDESSERVPROPCOM,
                CUSTO_PERC_CARDTRAB(CT.NRCARDAPIOTRAB, 'LIMPEZA') CUSTLIMERVPROPCOM,
                CUSTO_PERC_CARDTRAB(CT.NRCARDAPIOTRAB, NULL) CUSTTOTSERVPROPCOM
            FROM
                SERVICOPROPCOM SPC
                INNER JOIN DETALHESERVPROPCOM DSPC ON DSPC.NRDETALHESERVPROPCOM = SPC.NRDETALHESERVPROPCOM
                LEFT JOIN SERVICOPROPOSTA SP ON SPC.NRSERVICOPROPOSTA = SP.NRSERVICOPROPOSTA
                LEFT JOIN TIPOPUBLICO TP ON SPC.NRTIPOPUBLICO = TP.NRTIPOPUBLICO
                LEFT JOIN PADRAOQUALIDADE PQ ON PQ.NRPADRAOQUALIDADE = SPC.NRPADRAOQUALIDADE
                LEFT JOIN CARDAPIOTRAB CT ON CT.NRCARDAPIOTRAB = DSPC.NRCARDAPIOTRAB
            WHERE
                SPC.NRSERVICOPROPCOM = :NRSERVICOPROPCOM";
            
            $params = array(
                "NRSERVICOPROPCOM" => $nrServPropCom
            );

            $query = $this->entityManager->getConnection()->prepare($sql);
            $query->execute($params);
            $result = $query->fetchAll();

            return $result;

        } catch(Exception $e){
            throw $e;
        }
    }   

    /**
     * Rotina de notificar planejamento caso ocorra algum
     * erro durante a operação da proposta comercial e seus
     * derivados.
     * 
     * @param object $row
     * @param object $reqParams Request na função que 
     * originou a chamada.
     * @param object $respParams Response na função que 
     * originou a chamada.
     */
    public function notificarPlan($row, $reqParams = null, $respParams = null){
        $procResult = $this->solAltServPropCom($row);
        
        if($procResult['_procStatus']){
            if($reqParams){
                $this->tcoEmail->enviaEmail($reqParams, $respParams);
            }
        } else {
            return $procResult['_error'];
        }
    }
    
    public function enviaEmail($reqParams = null, $respParams = null){
        $this->tcoEmail->enviaEmail($reqParams, $respParams);
    }

    /**
     * Rotina que solicita alteração no serviço
     * da proposta comercial.
     * 
     * @param object $row
     * 
     * @return Error retorna erro em caso de falha
     * na execução da procedure.
     */
    public function solAltServPropCom ($row){
        try {
            //$row["TXHISTORICO"] =$this->buildHTMLHist($row);
            $row["TXHISTORICO"] = null;
            $procedure = new StoredProcedure($this->connection, 'SOL_ALT_SERVPROPCOM');
			$procedure->addParam(new Param('P_NRSERVICOPROPCOM', Param::PARAM_INPUT, $row['NRSERVICOPROPCOM'],                                    Param::PARAM_TYPE_INT));
			$procedure->addParam(new Param('P_IDORIGEM',         Param::PARAM_INPUT, $row['IDORIGEM'],                                            Param::PARAM_TYPE_STR));
			$procedure->addParam(new Param('P_IDDESTINO',        Param::PARAM_INPUT, $row['IDDESTINO'],                                           Param::PARAM_TYPE_STR));
			$procedure->addParam(new Param('P_IDMOTIVO',         Param::PARAM_INPUT, $row['IDMOTIVO'],                                            Param::PARAM_TYPE_STR));
			$procedure->addParam(new Param('P_DSOBSERVACAO',     Param::PARAM_INPUT, $row->has('DSOBSERVACAO') ? $row->get('DSOBSERVACAO') : ' ', Param::PARAM_TYPE_STR));
			$procedure->addParam(new Param('P_TXHISTORICO',      Param::PARAM_INPUT, $row['TXHISTORICO'],                                         Param::PARAM_TYPE_STR));
			$procedure->addParam(new Param('P_CDOPERADOR',       Param::PARAM_INPUT, $this->environment->getCdoperador(),                         Param::PARAM_TYPE_STR));
			$procedure->addParam(new Param('P_NRORG',            Param::PARAM_INPUT, $this->environment->getNrOrgTrab(),                          Param::PARAM_TYPE_STR));
            $result = $procedure->execute();
            
            return ["_procStatus"=>true];
        } catch (\Exception $e) {
            return [
                "_procStatus"=>false,
                "_error"=>new Error($e->getMessage(), -1)
            ];
        }
    }

    /**
     * Retorna HTML para texto do histórico de
     * notificação.
     *
     * @param object $row
     */
    public function buildHTMLHist($row){
	    $tpPratoQuant = $this->getTipoPratoEQuantidade($row);
	    $gramPorc = $this->getGramaturaEPorcionamento($row);
	    
	    $dsativcli = empty($tpPratoQuant["DSATIVCLI"]) ? '' : $tpPratoQuant["DSATIVCLI"]; 
	    $nmtipocontrato = empty($tpPratoQuant["NMTIPOCONTRATO"]) ? '' : $tpPratoQuant["NMTIPOCONTRATO"]; 
	    $nmpadraoqualidade = empty($tpPratoQuant["NMPADRAOQUALIDADE"]) ? '' : $tpPratoQuant["NMPADRAOQUALIDADE"]; 
	    $nmtipopublico = empty($tpPratoQuant["NMTIPOPUBLICO"]) ? '' : $tpPratoQuant["NMTIPOPUBLICO"]; 
	    $tiposprato = empty($tpPratoQuant["TIPOSPRATO"]) ? '' : $tpPratoQuant["TIPOSPRATO"];
        $qtsegsex = empty($tpPratoQuant["QTSEGSEX"]) ? '' : $tpPratoQuant["QTSEGSEX"]; 
        $qtsabado = empty($tpPratoQuant["QTSABADO"]) ? '' : $tpPratoQuant["QTSABADO"]; 
        $qtdomferi = empty($tpPratoQuant["QTDOMFERI"]) ? '' : $tpPratoQuant["QTDOMFERI"];
        $gramPorc = empty($gramPorc) ? '' : $gramPorc;
        
	    $html =    "<div class = 'content'>
                        <span class = 'header-item flex'>Escopo</span>
                        <div class = 'proprety-flex'>
                            <div class = 'flex'>
                                <span class = 'header-item'>Setor:</span>
                                <span class = 'value-item'></span>
                            </div>
                            
                            <div class = 'flex'>
                                <span class = 'header-item'>Ramo de Atividade:</span>
                                <span class = 'value-item'>".$dsativcli."</span>
                            </div>
                            
                            <div class = 'flex'>
                                <span class = 'header-item'>Tipo de Contrato:</span>
                                <span class = 'value-item'>".$nmtipocontrato."</span>
                            </div>
                            
                            <div class = 'flex'>
                                <span class = 'header-item'>Padrão de Qualidade:</span>
                                <span class = 'value-item'>".$nmpadraoqualidade."</span>
                            </div>
                            
                            <div class = 'flex'>
                                <span class = 'header-item'>Público:</span>
                                <span class = 'value-item'>".$nmtipopublico."</span>
                            </div>
                        </div>
                        
                        <span class = 'block-item'>Tipos de Prato:</span>
                        <span class = 'tipos-prato'>".$tiposprato."</span>
            
                        <span class = 'block-item'>Refeições Estimadas:</span>
                        <span class = 'tipos-prato'><b class = 'bold'>2ª a 6ª: </b> ".$qtsegsex."
                        <b class = 'space-left bold'> Sabádos: </b> ".$qtsabado."
                        <b class = 'space-left bold'> Domingos e Feriados: </b> ".$qtdomferi." </span>
            
                        <span class = 'block-item'>Porcionamento e Gramatura:</span>
                        <table>
                                ".$gramPorc."
                        </table>
            
                    </div>
                ";
	    
        return $html;
    }

    /**
     * Retorna tipo de prato e quantidade, dados
     * para procedure de solicitar alteração de
     * serviço.
     * 
     * @param object $row
     * @return array registro de tipo de prato.
     */
    private function getTipoPratoEQuantidade($row){
        $sql = "SELECT 
                SERVICOPROPCOM.NRSERVICOPROPCOM, 
                SERVICOPROPCOM.NRVERSAOPROPCOM, 
                SERVICOPROPCOM.NRSERVICOPROPOSTA, 
                SERVICOPROPCOM.NRTIPOPUBLICO,
                TIPOPUBLICO.NMTIPOPUBLICO,
                SERVICOPROPCOM.NRPADRAOQUALIDADE,
                CONFIGSERVPROPCOM.CDTIPOPRAT,
                TIPOPRAT.NMTIPOPRAT,
                SERVICOPROPCOM.QTSEGSEX,
                SERVICOPROPCOM.QTSABADO,
                SERVICOPROPCOM.QTDOMFERI,
                SERVICOPROPCOM.NRORG,
                PROPOSTACOMERCIAL.CDATIVCLI,
                PROPOSTACOMERCIAL.NRTIPOCONTRATO,
                PADRAOQUALIDADE.NMPADRAOQUALIDADE,
                ATIVCLIENTE.DSATIVCLI,
                SEGMERCADO.DSSEGMER,
                TIPOCONTRATO.NMTIPOCONTRATO 
            FROM SERVICOPROPCOM 
                INNER JOIN CONFIGSERVPROPCOM ON CONFIGSERVPROPCOM.NRDETALHESERVPROPCOM = NVL( SERVICOPROPCOM.NRDETSERVPROPCOMPLAN, SERVICOPROPCOM.NRDETALHESERVPROPCOM ) 
                INNER JOIN VERSAOPROPCOM ON SERVICOPROPCOM.NRVERSAOPROPCOM = VERSAOPROPCOM.NRVERSAOPROPCOM 
                INNER JOIN PROPOSTACOMERCIAL ON PROPOSTACOMERCIAL.NRPROPOSTACOMERCIAL = VERSAOPROPCOM.NRPROPOSTACOMERCIAL 
                LEFT JOIN PADRAOQUALIDADE ON PADRAOQUALIDADE.NRPADRAOQUALIDADE = SERVICOPROPCOM.NRPADRAOQUALIDADE 
                LEFT JOIN TIPOPRAT ON TIPOPRAT.CDTIPOPRAT = CONFIGSERVPROPCOM.CDTIPOPRAT 
                LEFT JOIN SEGMERCADO ON PROPOSTACOMERCIAL.CDSEGMER = SEGMERCADO.CDSEGMER 
                LEFT JOIN ATIVCLIENTE  ON PROPOSTACOMERCIAL.CDATIVCLI = ATIVCLIENTE.CDATIVCLI 
                LEFT JOIN TIPOCONTRATO ON PROPOSTACOMERCIAL.NRTIPOCONTRATO = TIPOCONTRATO.NRTIPOCONTRATO 
                LEFT JOIN TIPOPUBLICO ON TIPOPUBLICO.NRTIPOPUBLICO = SERVICOPROPCOM.NRTIPOPUBLICO 
            WHERE 
                SERVICOPROPCOM.NRSERVICOPROPOSTA =:NRSERVICOPROPOSTA AND 
                SERVICOPROPCOM.NRSERVICOPROPCOM =:NRSERVICOPROPCOM";

        $params = array(
            "NRSERVICOPROPOSTA" => $row->get("NRSERVICOPROPOSTA"),
            "NRSERVICOPROPCOM" => $row->get("NRSERVICOPROPCOM")
        );
	    $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        $result = $query->fetchAll();
        $tpPratoQuant = $this->buildStringTipoPratoQuant($result);
        return $tpPratoQuant;
    }
    
    /**
     * Retorna tipo de prato e quantidade, dados
     * para procedure de solicitar alteração de
     * serviço.
     * 
     * @param object $row
     * @return array registro de tipo de prato.
     */
    private function getGramaturaEPorcionamento($row){
        $sql = "SELECT 
                SPC.NRSERVICOPROPCOM, GTP.CDGRTIPOPR, GTP.NMGRTIPOPR,
                P.NMPORCIONAMENTO, P.CDPORCIONAMENTO, NVL(GCB.VRGRAMATURA, '') AS VRGRAMATURA, 
                NVL(GCB.IDTIPOGRAMATURA, '') AS IDTIPOGRAMATURA, SPC.NRORG 
            FROM 
                SERVICOPROPCOM SPC 
                LEFT JOIN CONFGRUPOTPPROPCOM CGPC ON SPC.NRDETALHESERVPROPCOM = CGPC.NRDETALHESERVPROPCOM 
                LEFT JOIN GRAMATURACARDBASE GCB ON GCB.NRGRAMATURACARDBASE = CGPC.NRGRAMATURACARDBASE 
                LEFT JOIN PORCIONAMENTO P ON P.NRPORCIONAMENTO = CGPC.NRPORCIONAMENTO
                LEFT JOIN GRTIPOPR GTP ON GTP.CDGRTIPOPR = CGPC.CDGRTIPOPR
            WHERE 
                SPC.NRSERVICOPROPOSTA =:NRSERVICOPROPOSTA AND SPC.NRSERVICOPROPCOM =:NRSERVICOPROPCOM";

        $params = array(
            "NRSERVICOPROPOSTA" => $row->get("NRSERVICOPROPOSTA"),
            "NRSERVICOPROPCOM" => $row->get("NRSERVICOPROPCOM")
        );

	    $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        $result = $query->fetchAll();
        $gramPorc = $this->buildHtmlGramPorc($result);
        return $gramPorc;
    }

    /**
     * Retorna string de tipo de prato e quantidade
     * formatada para ser usada na procedure de 
     * alteração de serviço da proposta comercial.
     * 
     * @param object $result Resultado da consulta na
     * função 'getTipoPratoEQuantidade'
     * 
     * @return string String formatada.
     */
    private function buildStringTipoPratoQuant($result){
        $tiposPrato = ""; $qtsegsex = ""; $qtsabado = ""; $qtdomferi = "";
        $padraoqualidade = ""; $dssegmer = ""; $nmtipocontrato = ""; $dsativcli = "";
        $tpPratoQuant = "";
        if(!empty($result)){
            
            $length = sizeof($result)-1;
            
            foreach($result as $key => &$data){
                if($key != $length)
                    $tiposPrato = $tiposPrato.$data["NMTIPOPRAT"].", ";
                else
                    $tiposPrato = $tiposPrato.$data["NMTIPOPRAT"];
            }
            
            $tpPratoQuant = array(
                "TIPOSPRATO" => $tiposPrato,
                "QTSEGSEX" => $result[0]["QTSEGSEX"],
                "QTSABADO" => $result[0]["QTSABADO"],
                "QTDOMFERI" => $result[0]["QTDOMFERI"],
                "NMTIPOCONTRATO" => $result[0]["NMTIPOCONTRATO"],
                "DSATIVCLI" => $result[0]["DSATIVCLI"],
                "NMTIPOPUBLICO" => $result[0]["NMTIPOPUBLICO"],
                "NMPADRAOQUALIDADE" => $result[0]["NMPADRAOQUALIDADE"]
            );
            
        }
        
        return $tpPratoQuant;
    }
    
    /**
     * Retorna string HTML formatada de gramatura 
     * e porcionamento para ser usada na procedure
     * alteração de serviço da proposta comercial.
     * 
     * @param object $result Resultado da consulta na função
     * 'getGramaturaEPorcionamento'.
     * 
     * @return string HTML formatado.
     */
    private function buildHtmlGramPorc($result){
        $html = "";
        foreach ($result as $key => &$data) {
            $html .= "<tr>"; 
            $html .= "<td class = 'space-right'>".$data["NMGRTIPOPR"]."</td>";
            $html .= "<td class = 'space-right'>".$data["NMPORCIONAMENTO"]."</td>";
            $html .= "<td class = 'space-right'>".$data["VRGRAMATURA"]."</td>";
            $html .= "<td class = 'space-right'>".$data["IDTIPOGRAMATURA"]."</td>";
            $html .= "</tr>";
        }
        return $html;
    }   
}